# Copyright (c) 2005, Henry Ludemann
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
#  * Neither the name of the organization nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

class ArgumentError(Exception):
    pass

class Argument(object):
    "Class to hold information about an argument"
    def __init__(self, name, validate):
        """Construct an Argument object

        name -- This is the name that will be used on the command line; eg:
            Usage: SomeCommand.py [SOME_NAME]
            It should be made up of capital letters and underscores (no 
            spaces).
        """
        self._name = name
        self._validate = validate 

    name = property(lambda self:self._name, doc="This is the name of the " \
        "argument that will be used on the command line " \
        "(eg: blah.py SOME_ARG)")

    def get_value(self, value):
        """Is a value valid for a particular argument

        Return None if the value isn't valid, otherwise an object."""
        result = None
        try:
            result = self._validate(value)
        except Exception, err:
            raise ArgumentError("Invalid value for argument %s - %s" % (self.name, err))
        return result
