#! /usr/bin/env python

import sys
import unittest

import cmdline.test.testargument
import cmdline.test.testcommand
import cmdline.test.testconfig

modules = [cmdline.test.testargument,
        cmdline.test.testcommand,
        cmdline.test.testconfig,
        ]

def run_tests(name="", quiet=False, verbose=False):
    if quiet:
        verbosity = 0
    elif verbose:
        verbosity = 2
    else:
        verbosity = 1

    loader = unittest.TestLoader()
    if name:
        suite = loader.loadTestsFromName(name)
    else:
        tests = []
        for module in modules:
            tests.extend(loader.loadTestsFromModule(module))
        suite = unittest.TestSuite(tests)

    runner = unittest.TextTestRunner(verbosity=verbosity)
    result = runner.run(suite)
    sys.exit(not result.wasSuccessful())

command = cmdline.Command(run_tests, "Run the cmdline unit tests.")
command.add_flag('v', 'verbose', 'Verbose output', 'verbose')
command.add_argument('TEST_NAME', 'An optional test name, specifying what test to run', 'name')
command.invoke()
