#!env python
import cmdline
import logging
import os.path
import sys

from pycro.backend.function import Call
import pycro.backend.codegen as cgen
import pycro.common.config as conf
import pycro.frontend.python.test.run
import pycro.frontend.python.compiler as comp

def _compile(code, config):
    codegen = cgen.CodeGen(config)
    compiler = comp.Compiler(codegen, config)
    compiler.decode_module(code)
    return codegen.create_output()

def _get_default_config():
    config = conf.Config()
    config.display_version = False
    config.filename = ""
    config.verbose = False
    
    pycro.backend.setup_config(config)
    pycro.frontend.python.setup_config(config)
    return config

def append_path(list, value):
    dir = os.path.normpath(os.path.join(os.getcwd(), value))
    if not os.path.isdir(dir):
        dir = value

    if not os.path.isdir(dir):
        raise ValueError("Missing path '%s'!" % value)

    list.append(dir)
    return list

def version():
    return "0.0 devel"

def main():
    cfg = _get_default_config()
    command = cmdline.ConfigSetter(cfg, "Compile python code to machine code")
    command.add_flag('I', 'include', "Set the python incude path",
            "python.include_path", lambda i:append_path(cfg.python.include_path, i))
    command.add_flag('v', 'verbose', "Verbose logging", "verbose")
    command.add_argument('FILENAME', "Python file to compile", "filename")
    command.add_flag(None, 'overflow-checks', "Enable debug overflow checks in the backend", "i386.overflow_checks")
    command.add_flag(None, 'bounds-checks', "Enable debug list bound checks in the backend", "i386.bounds_checks")
    command.add_flag("g", 'debug-info', "Create debbuging information", "debug_info")
    command.add_flag(None, 'version', "Display version information", "display_version")

    result = command.invoke()
    if result is not None:
        sys.exit(result)

    if cfg.display_version:
        print version()
        sys.exit(0)

    if not cfg.filename:
        name = "<stdin>"
        output_filename = "output.com"

        sys.stdout.write("    *****************************************\n")
        sys.stdout.write("    Pycro version %s\n" % version())
        sys.stdout.write("\n")
        sys.stdout.write("    Type the python code to be compiled. When\n")
        sys.stdout.write("    complete, press Ctrl^Z - Enter to compile\n")
        sys.stdout.write("    the code.\n")
        sys.stdout.write("\n")
        sys.stdout.write("    The output file will be %s.\n" % output_filename)
        sys.stdout.write("    *****************************************\n")

        input = sys.stdin.read()
    else:
        name = cfg.filename
        input = file(name, "r").read()
        output_filename = "%s.com" % os.path.splitext(os.path.split(name)[1])[0]

        # Add the directory to the search path
        append_path(cfg.python.include_path, os.path.dirname(cfg.filename))

    if cfg.verbose:
        logging.basicConfig(level=logging.INFO)
    logging.info("Search path: %s", cfg.python.include_path)

    code = compile(input, name, "exec")
    output = _compile(code, cfg)

    output_file = file(output_filename, "wb")
    for byte in output:
        output_file.write(chr(byte))
    output_file.close()

if __name__ == "__main__":
    main()
