import pycro.backend.function as func
import pycro.backend.variable as var

class _CallableBuilder:
    """ Class to call functions to generate code on demand """
    def __init__(self, codegen):
        self._codegen = codegen
        self._uncompiled = []
        self._builtins = {}

    def add_reference(self, name, builder, num_args):
        if name not in self._builtins:
            self._builtins[name] = func.Function(self._codegen, name)
            self._uncompiled.append((self._builtins[name], builder, num_args))

        self._builtins[name].add_reference()

    def __call__(self):
        while len(self._uncompiled) > 0:
            (function, builder, num_args) = self._uncompiled.pop()

            function.begin(num_args)
            variables = [var.StackVariable(self._codegen, num_args - i) for i in range(num_args)]
            builder(self._codegen, function, variables)
            function.end()

def builder(args):
    """ Decorator to mark a function as being able to generate output.

    Adds an 'add_reference' method to the function, allowing it to be
    called to add a reference to the compiled output.
    """
    assert isinstance(args, list), "Builder decorator function takes a list of argument names!"
    def generator(func):
        assert callable(func)
        def add_reference(codegen):
            builder = codegen.get_post_step(_CallableBuilder)
            builder.add_reference(func.__name__, func, len(args))
            return len(args)
        func.add_reference = add_reference
        return func
    return generator

