#! env python
import unittest

import pycro.backend.test.comfile as cfile
import pycro.backend.codegen as cgen
import pycro.backend.control.label as lbl
import pycro.backend.operations.softwareinterrupt as si

class TestLabel(unittest.TestCase):
    def test_jump(self):
        codegen = cgen.CodeGen()
        label = lbl.Label(codegen)

        label.jump()
        # This shouldn't be executed...
        codegen.push_data(3)
        si.SoftwareInterrupt(codegen).exit()

        label.resolve()
        codegen.push_data(5)
        si.SoftwareInterrupt(codegen).exit()

        self.assertEqual(5, cfile.run(codegen.create_output())[0])

    def test_no_jump_if_false(self):
        codegen = cgen.CodeGen()
        label = lbl.Label(codegen)

        codegen.push_data(0)
        label.jump_if_true()
        # This should be executed...
        codegen.push_data(3)
        si.SoftwareInterrupt(codegen).exit()

        label.resolve()
        # We shouldn't have jumped here...
        codegen.push_data(5)
        si.SoftwareInterrupt(codegen).exit()

        self.assertEqual(3, cfile.run(codegen.create_output())[0])

    def test_jump_if_true(self):
        codegen = cgen.CodeGen()
        label = lbl.Label(codegen)

        codegen.push_data(1)
        label.jump_if_true()
        # This should not be executed...
        codegen.push_data(3)
        si.SoftwareInterrupt(codegen).exit()

        label.resolve()
        # We should have jumped here...
        codegen.push_data(5)
        si.SoftwareInterrupt(codegen).exit()

        self.assertEqual(5, cfile.run(codegen.create_output())[0])

    def test_jump_if_false(self):
        codegen = cgen.CodeGen()
        label = lbl.Label(codegen)

        codegen.push_data(0)
        label.jump_if_false()
        # This should not be executed...
        codegen.push_data(3)
        si.SoftwareInterrupt(codegen).exit()

        label.resolve()
        # We should have jumped here...
        codegen.push_data(5)
        si.SoftwareInterrupt(codegen).exit()

        self.assertEqual(5, cfile.run(codegen.create_output())[0])

    def test_no_jump_if_true(self):
        codegen = cgen.CodeGen()
        label = lbl.Label(codegen)

        codegen.push_data(1)
        label.jump_if_false()
        # This should be executed...
        codegen.push_data(3)
        si.SoftwareInterrupt(codegen).exit()

        label.resolve()
        # We shouldn't have jumped here...
        codegen.push_data(5)
        si.SoftwareInterrupt(codegen).exit()

        self.assertEqual(3, cfile.run(codegen.create_output())[0])

    def test_error_on_different_stack_sizes(self):
        codegen = cgen.CodeGen()
        label = lbl.Label(codegen)

        label.jump()
        codegen.push_data(3)
        self.assertRaises(lbl.DifferingStackSizesError, label.resolve)

    def test_error_on_unresolved_label(self):
        codegen = cgen.CodeGen()
        label = lbl.Label(codegen)
        label.jump()
        self.assertRaises(lbl.UnresolvedLabelError, codegen.create_output)


if __name__ == "__main__":
    unittest.main()
