import pycro.backend.codegen as cgen
from pycro.backend.builder import builder
import pycro.backend.operations.integer as integer
import pycro.backend.operations.softwareinterrupt as si
import pycro.backend.variable as var

@builder(args=[])
def get_milliseconds(codegen, function, variables):
    # Interrupt 10h where AH=2c gives the system time
    # such that CH=hour CL=minute DH=second DL=1/100 seconds
    #
    # We only conver the seconds & hundreths of a second to
    # milliseconds (because in 16bits that's all we can store)

    # NOTE: Could we hook into interrupt 70h to get the 1khz timer?
    si.SoftwareInterrupt(codegen).trigger_dos_interrupt(0x2c)
    codegen.push(cgen.Register.DX)
    sec_ms = var.StackVariable(codegen)

    # Use the minutes, seconds, and milliseconds to create a 'milliseconds' 
    # counter. First push the milliseconds.
    sec_ms.push()
    codegen.push_data(0x00ff)
    integer.Integer(codegen).binary_and()
    codegen.push_data(10)
    integer.Integer(codegen).multiply()

    # Now push the seconds (converted to milliseconds)
    sec_ms.push()
    codegen.push_data(0xff00)
    integer.Integer(codegen).binary_and()
    codegen.push_data(8)
    integer.Integer(codegen).shift_right()
    codegen.push_data(1000)
    integer.Integer(codegen).multiply()

    # Now add the two results together, and return
    # that result.
    integer.Integer(codegen).add()
    function.ret()

@builder(args=["milliseconds"])
def sleep(codegen, function, variables):
    # FIXME The stack is stuffing up... this 'push' shouldn't be
    # necessary
    #
    # Could we use int 15h AH=86h?
    codegen.push_data(0)
    function.ret()
