from pycro.backend.builder import builder
from pycro.backend.codegen import Register, Register8Bit, Mod, SegmentRegister, R_M, BASE_ADDRESS
import pycro.backend.control.label as lbl
from pycro.backend.variable import GlobalVariable

class _State:
    """
    Object to keep track of whether the graphics subsystem has been used.
    """

    def __init__(self, codegen):
        self._codegen = codegen
        self.initialised = GlobalVariable(self._codegen, False)
        self._offset = None

    def __call__(self):
        if self._offset is None:
            self._offset = self._codegen.get_code_offset() + BASE_ADDRESS
            self._codegen.add_instr(0)
            self._codegen.add_instr(0)
        self.initialised.resolve(self._offset)

@builder(args=[])
def init(codegen, function, variables):
    codegen.push_data(1)
    codegen.get_post_step(_State).initialised.pop()

    XOR = 0x31
    codegen.add_rm_instr(XOR, Mod.DIRECT_REGISTER, Register.AX, Register.AX)

    MOV_reg8_imm8 = 0xb0
    codegen.add_instr(MOV_reg8_imm8 + Register8Bit.AL)
    codegen.add_instr(0x13) # Mode 13 (320 x 200 x 8bit)

    SOFTWARE_INTERRUPT = 0xcd
    codegen.add_instr(SOFTWARE_INTERRUPT)
    codegen.add_instr(0x10) # Video interupt

    codegen.push_data(0)
    function.ret()

@builder(args=["x", "y", "colour"])
def set_pixel(codegen, function, variables):
    assert len(variables) == 3
    x = variables[0]
    y = variables[1]
    colour = variables[2]

    # es = 0xa000
    codegen.push_data(0xa000)
    codegen.pop(Register.AX)
    MOV_SEG = 0x8E
    codegen.add_rm_instr(MOV_SEG, Mod.DIRECT_REGISTER, SegmentRegister.ES, Register.AX)

    # di = y * 320 + x
    codegen.push_data(320)
    y.push()
    codegen.pop(Register.AX)
    codegen.pop(Register.BX)
    codegen.add_rm_instr(0xF7, Mod.DIRECT_REGISTER, 4, Register.BX)
    codegen.push(Register.AX)
    ADD = 0x01
    x.push()
    codegen.add_binary_instr(ADD)
    codegen.pop(Register.DI)

    # ax = colour
    colour.push()
    codegen.pop(Register.AX)
    
    STOSB = 0xAA
    codegen.add_instr(STOSB)

    codegen.push_data(0)
    function.ret()

@builder(args=[])
def refresh(codegen, function, variables):
    # Don't attempt to do any screen syncs at this stage...
    codegen.push_data(0)
    function.ret()

@builder(args=[])
def close(codegen, function, variables):
    codegen.get_post_step(_State).initialised.push()
    block = lbl.Label(codegen)
    block.jump_if_false()

    MOV_reg8_imm8 = 0xb0
    codegen.add_instr(MOV_reg8_imm8 + Register8Bit.AH)
    codegen.add_instr(0x0)
    codegen.add_instr(MOV_reg8_imm8 + Register8Bit.AL)
    codegen.add_instr(0x3) # Text mode

    SOFTWARE_INTERRUPT = 0xcd
    codegen.add_instr(SOFTWARE_INTERRUPT)
    codegen.add_instr(0x10) # Video interupt

    block.resolve()

    codegen.push_data(0)
    function.ret()

