import pycro.backend.codegen as cgen
import pycro.backend.function as func
import pycro.backend.operations.asserter as asrt
import pycro.backend.operations.integer as intgr
import pycro.backend.operations.softwareinterrupt as si
import pycro.backend.variable as var

class _Allocator:
    """ Class to handle allocations """
    def __init__(self, codegen):
        self._codegen = codegen
        self._func = func.Function(codegen, "<allocate>")
        self._alloc_var = None
        codegen.add_final_step(self._set_alloc_addr)
        self._have_created_allocator = False

    def _create_alloc_function(self):
        self._func.begin(1)
        size = var.StackVariable(self._codegen, 1)
        alloc_addr = var.GlobalVariable(self._codegen, False)

        # Detect whether we will enter into the stack region of memory. This
        # detection is only for the allocation; if the stack goes into the
        # allocation region that isn't detected.
        MOV = 0x8B
        self._codegen.add_rm_instr(MOV, cgen.Mod.DIRECT_REGISTER, cgen.Register.AX, cgen.Register.SP)
        self._codegen.push(cgen.Register.AX)
        size.push()
        SUB = 0x29
        self._codegen.add_binary_instr(SUB)
        alloc_addr.push()
        intgr.Integer(self._codegen).unsigned_lesser()
        asrt.assertTrue(self._codegen, None, None, "Out of memory!")

        # Get the actual return address
        alloc_addr.push()
        alloc_addr.push()
        size.push()
        ADD = 0x01
        self._codegen.add_binary_instr(ADD)
        alloc_addr.pop()

        self._func.ret()
        self._func.end()

        # Create a variable to hold the current 'end'.
        self._alloc_var = self._codegen.get_code_offset()
        offset = self._alloc_var + cgen.BASE_ADDRESS
        alloc_addr.resolve(offset)
        self._codegen.add_instr(0)
        self._codegen.add_instr(0)

    def _set_alloc_addr(self):
        # We start allocating memory immediately after the generated code
        free_addr = self._codegen.get_free_offset() + cgen.BASE_ADDRESS
        msb = free_addr >> 8
        lsb = free_addr & 0xff
        self._codegen.set_instr(self._alloc_var, lsb)
        self._codegen.set_instr(self._alloc_var + 1, msb)

    def allocate(self):
        """ \see pycro.backed.operators.allocator.allocate """
        # To use the standard 'Call' class, we need to push the
        # function before the arguments. Reorganise the items on the
        # stack...
        self._codegen.pop(cgen.Register.AX)
        self._func.add_reference()
        self._codegen.push(cgen.Register.AX)
        func.Call(self._codegen).call(1)

    def __call__(self):
        if not self._have_created_allocator:
            self._create_alloc_function()
            self._have_created_allocator = True

def allocate(codegen):
    """ Allocate an amount of items given the value on top of the stack.
    
        The allocated address is pushed on the stack. """
    codegen.get_post_step(_Allocator).allocate()

