import pycro.backend.codegen as cgen
import pycro.backend.control.label as lbl
import pycro.backend.function as func
import pycro.backend.operations.softwareinterrupt as si
import pycro.backend.operations.string as string
import pycro.backend.sourcelookup as sl
import pycro.backend.variable as var
import pycro.frontend.python.compiler as cmplr

class _Asserter:
    """ Class to implement an assertion routine """
    def __init__(self, codegen):
        self._codegen = codegen
        self._function = func.Function(self._codegen, "<assert>")
        self._strings = {}
        self._have_generated = False

    def __call__(self):
        if not self._have_generated:
            self._function.begin(3)

            # This is currently an overhead on every program, whether
            # the use the graphics subsystem or not...
            from pycro.backend.lib.system.drawing.window import close
            close.add_reference(self._codegen)
            func.Call(self._codegen).call(0)
            self._codegen.pop_data()

            compiler = cmplr.Compiler(self._codegen)
            compiler.decode(self._function, self._assert.func_code)
            self._function.end()
            self._have_generated = True

    def _assert(file, line, message):
        from sys import exit

        # We have a special case for '-1' to handle out of memory errors;
        # in these situations we don't want to allocate even more memory
        # by creating a string!
        line_no = "-1"
        if line != -1:
            line_no = str(line)
        print file, '(', line_no, '): AssertionError:', message
        exit(1)

    def failure(self, file, line, message):
        self._function.add_reference()
        string.add_string_reference(self._codegen, file)
        self._codegen.push_data(line)
        message.push()
        func.Call(self._codegen).call(3)

        # We need to pop the return value
        self._codegen.pop_data(var._VARIABLE_SIZE)

def assertTrue(codegen, file, line, message):
    """ Assert that the item on top of the stack is true """
    label = lbl.Label(codegen)
    label.jump_if_false()
    string.add_string_reference(codegen, message)
    message = var.StackVariable(codegen)
    failure(codegen, message, file, line)
    # Remove the message variable
    codegen.pop_data()
    label.resolve()

def failure(codegen, message, file=None, line=-1):
    """ Raise an assertion error """
    if file is None:
        (file, line, line_contents) = sl.SourceReference.instance(codegen).get_source_location()
        if file is None:
            file = "<unknown>"
            line = -1
        if not message and line_contents:
            message = line_contents.strip()

    codegen.get_post_step(_Asserter).failure(file, line, message)

