from pycro.backend.codegen import Register, Register8Bit

class SoftwareInterrupt:
    def __init__(self, codegen):
        self._codegen = codegen

    def exit(self):
        self._codegen.pop(Register.AX) # Pop the exit status

        TERMINATE_WITH_RETURN_CODE = 0x4c
        self.trigger_dos_interrupt(TERMINATE_WITH_RETURN_CODE)

    def trigger_dos_interrupt(self, interrupt):
        MOV_reg8_imm8 = 0xb0
        self._codegen.add_instr(MOV_reg8_imm8 + Register8Bit.AH)
        self._codegen.add_instr(interrupt)

        SOFTWARE_INTERRUPT = 0xcd
        self._codegen.add_instr(SOFTWARE_INTERRUPT) # Trigger software interrupt
        self._codegen.add_instr(0x21) # DOS interupt

    def print_chr(self):
        """ Pop an integer from the stack, and treat it as a character 
            to be printed. """
        self._codegen.pop(Register.DX) # Move character to print into dx

        WRITE_CHAR_TO_STDOUT = 0x02
        self.trigger_dos_interrupt(WRITE_CHAR_TO_STDOUT)
