#!env python
import unittest

import pycro.backend.codegen as cgen
import pycro.backend.operations.asserter as asrt
import pycro.backend.operations.softwareinterrupt as si
from pycro.backend.test.comfile import run

class TestAssert(unittest.TestCase):
    def test_failing_assert(self):
        codegen = cgen.CodeGen()
        codegen.push_data(1)
        asrt.assertTrue(codegen, "somefile.py", 23, "hey, I failed!")
        codegen.push_data(0)
        si.SoftwareInterrupt(codegen).exit()

        result = run(codegen.create_output())
        self.assertEqual(1, result[0])
        self.assertEqual("somefile.py ( 23 ): AssertionError: hey, I failed!\n", result[1])

    def test_passing_assert(self):
        codegen = cgen.CodeGen()
        codegen.push_data(0)
        asrt.assertTrue(codegen, "somefile", 23, "hey, I shouldn't have failed!")
        codegen.push_data(0)
        si.SoftwareInterrupt(codegen).exit()

        result = run(codegen.create_output())
        self.assertEqual(0, result[0])
        self.assertEqual("", result[1])

if __name__ == "__main__":
    unittest.main()
