#!env python
import unittest
from pycro.backend.operations.string import add_string_reference, \
    print_string
from pycro.backend.operations.softwareinterrupt import SoftwareInterrupt
from pycro.backend.codegen import CodeGen
from pycro.backend.test.comfile import write_and_run, run

class TestString(unittest.TestCase):
    def test_define(self):
        """ Test that we can define a string (but don't do anything with it) """
        
        generator = CodeGen()
        add_string_reference(generator, "bob")
        generator.push_data(1)
        SoftwareInterrupt(generator).exit()

        self.assertEqual(1, write_and_run(generator.create_output()))

    def test_print(self):
        """ Test that we can print strings """
        
        generator = CodeGen()
        add_string_reference(generator, "rabbit")
        print_string(generator)
        generator.push_data(0)
        SoftwareInterrupt(generator).exit()

        self.assertEqual('rabbit', run(generator.create_output())[1])

    def test_stack_cleanup(self):
        
        generator = CodeGen()
        self.assertEqual(0, generator.get_stack_size())
        add_string_reference(generator, "rabbit")
        self.assertEqual(2, generator.get_stack_size())
        print_string(generator)
        self.assertEqual(0, generator.get_stack_size())

if __name__ == "__main__":
    unittest.main()
