import logging
import os.path

import pycro.backend.codegen as cgen

log = logging.getLogger()

class SourceReference:
    @staticmethod
    def instance(codegen):
        if not hasattr(codegen, "source_reference"):
            codegen.source_reference = SourceReference(codegen)
        return codegen.source_reference

    def __init__(self, codegen):
        self._last_line = None
        self._last_file = None
        self._last_line_contents = None
        self._codegen = codegen

        if self._codegen.config.debug_info:
            codegen.add_final_step(self._write_database)
        self._items = []

        self._cached_file = None
        self._cached_contents = ""

    def _sort_by_file_and_line(self, a, b):
        if a[0] > b[0]:
            return 1
        if b[0] > a[0]:
            return -1
        if a[1] > b[1]:
            return 1
        if b[1] > a[1]:
            return -1
        return 0

    def _write_database(self):
        output = file("output.txt", "w")
        
        # Sort by filename and line number
        self._items.sort(self._sort_by_file_and_line)
        current_filename = None
        for filename, line_number, address in self._items:
            if current_filename != filename:
                output.write("%s\n" % filename)
                current_filename = filename
            output.write("%s\t%s\n" % (hex(address), self._readline(filename, line_number)))

    def add_source_reference(self, filename, line_number):
        """
        Reference the current instructions with a given source location.
        """
        if self._last_line != line_number or filename != self._last_file:
            address = self._codegen.get_code_offset() + cgen.BASE_ADDRESS
            self._items.append((filename, line_number, address))
            self._last_file = filename
            self._last_line = line_number

            if log.isEnabledFor(logging.INFO):
                log.info("%s[%i]\t%s\n" % (filename, line_number, self._readline(filename, line_number)))

    def _readline(self, filename, line_number):
        line = ""
        if filename is not None and os.path.exists(filename) and line_number > 0:
            if filename != self._cached_file:
                source = file(filename, "r")
                self._cached_contents = source.readlines()
                source.close()
                self._cached_file = filename

            # Empty files have added source references of '1', but 
            # there is no first line.
            if line_number < len(self._cached_contents):
                line = self._cached_contents[line_number - 1]
        return line

    def get_source_location(self):
        return (self._last_file, self._last_line, self._readline(self._last_file, self._last_line))

def add_source_reference(codegen, filename, line_number):
    SourceReference.instance(codegen).add_source_reference(filename, line_number)
