#!env python
import unittest
from pycro.backend.codegen import CodeGen
from pycro.backend.variable import StackVariable, GlobalVariable
from pycro.backend.operations.softwareinterrupt import SoftwareInterrupt
from pycro.backend.test.comfile import write_and_run
from pycro.backend.operations.integer import Integer

class VariableTests:
    """ Set of generic tests for both the global & local variables """
    def test_declare(self):
        
        generator = CodeGen()
        generator.push_data(1)
        var_a = self.Variable(generator)
        var_a.push()
        SoftwareInterrupt(generator).exit()
        self.assertEqual(1, write_and_run(generator.create_output()))

        generator = CodeGen()
        generator.push_data(0)
        var_a = self.Variable(generator)
        var_a.push()
        SoftwareInterrupt(generator).exit()
        self.assertEqual(0, write_and_run(generator.create_output()))

    def test_declare_two(self):
        # Test declaring two operations, and make sure we can use the value
        # of the first one
        
        generator = CodeGen()
        generator.push_data(1)
        var_a = self.Variable(generator)
        generator.push_data(0)
        var_b = self.Variable(generator)
        var_a.push()
        SoftwareInterrupt(generator).exit()
        self.assertEqual(1, write_and_run(generator.create_output()))

    def test_var_operation(self):
        # Initialise a variable from the result of an operation
        
        generator = CodeGen()
        generator.push_data(5)
        generator.push_data(2)
        Integer(generator).add()
        var_a = self.Variable(generator)

        # Push something else onto the stack to make sure the
        # variable is correctly put on afterwards
        generator.push_data(3)
        var_a.push()
        SoftwareInterrupt(generator).exit()
        self.assertEqual(7, write_and_run(generator.create_output()))

    def test_pop(self):
        """ Initialise a variable, then change its value """
        
        generator = CodeGen()
        # Initialise the variable to 7
        generator.push_data(7)
        var_a = self.Variable(generator)

        # Change the variable to 9
        generator.push_data(9)
        var_a.pop()

        # Add some data to the stack
        generator.push_data(6)

        # Make sure the variables value is 9
        var_a.push()
        SoftwareInterrupt(generator).exit()
        self.assertEqual(9, write_and_run(generator.create_output()))

class TestStackVariables(VariableTests, unittest.TestCase):
    Variable = StackVariable

class TestGlobalVariables(unittest.TestCase):
    """ Temporary test class to specifically test global variables.

        Once stack variables allocate their own memory (on the stack),
        re-use the stack variable test cases. """
    def test_declare(self):
        
        generator = CodeGen()
        var_a = GlobalVariable(generator)
        generator.push_data(1)
        var_a.pop()
        var_a.push()
        SoftwareInterrupt(generator).exit()
        self.assertEqual(1, write_and_run(generator.create_output()))

        generator = CodeGen()
        var_a = GlobalVariable(generator)
        generator.push_data(0)
        var_a.pop()
        var_a.push()
        SoftwareInterrupt(generator).exit()
        self.assertEqual(0, write_and_run(generator.create_output()))

if __name__ == '__main__':
    unittest.main()

