
import pycro.frontend.python.opcode as ops

def opcode(opcode_name):
    """ 
    Create a decorator for marking an opcode handler.
    
    The handler will be called with the arguments
    (state, codegen, code). The current opcode will still
    be in the code object, but will be automatically removed
    after the handler returns.
    """
    def decorator(func):
        ops.Opcodes.add_opcode(func, opcode_name)
        return func
    return decorator

def opcode_with_index(opcode_name):
    """
    Create a decorator for an opcode handler for an opcode that has a 2 byte index.
    
    The handler will be called with the arguments 
    (state, codegen, index, code, byte_code_counter). The current opcode will still
    be in the code object, but will be automatically removed
    after the handler returns.
    """ 
    def decorator(func):
        def call_with_index(state, codegen, code, byte_code_counter):
            index = code.pop_index()
            func(state, codegen, index, code, byte_code_counter)

        ops.Opcodes.add_opcode(call_with_index, opcode_name)
        return func

    return decorator

