import dis

import pycro.backend.control.label as lbl
import pycro.backend.operations.integer as integer
import pycro.backend.operations.list as lst
import pycro.backend.variable as var
from pycro.frontend.python.opcode import Opcodes
from pycro.frontend.python.opcodes import opcode, opcode_with_index


@opcode('GET_ITER')
def get_iterator(state, codegen, code, bcc):
    assert code.peek(0) == dis.opname.index('FOR_ITER')

    # Create a counter variable on the stack...
    codegen.push_data(0)
    #i = var.StackVariable(self._codegen)

@opcode_with_index('FOR_ITER')
def for_iterator(state, codegen, offset, code, bcc):
    loop_end = lbl.Label(codegen)

    # At this stage, we've got the list and the counter on
    # the stack. We'll make sure we haven't reached the end
    # of the list, then push the next item onto the stack.
    index = var.StackVariable(codegen)
    items = var.StackVariable(codegen, 1)

    # Jump to the end of the loop if we have reached
    # the end of the items.
    index.push()
    items.push()
    lst.length(codegen)
    integer.Integer(codegen).lesser()
    loop_end.jump_if_false()

    # Push the 'entry' variable (leaving it on the stack).
    items.push()
    index.push()
    lst.get_item(codegen)

    # Increment the counter. Note that we have to do this before
    # we decode the rest, as it will have 'continue' statements,
    # and we need 'i' to be incremented.
    index.push()
    codegen.push_data(1)
    integer.Integer(codegen).add()
    index.pop()

    # Decode the for loop code
    for_loop_code = code.remove(0, offset)
    Opcodes(codegen).decode(state, for_loop_code)

    # Cleanup the loop (both the index and the list reference)
    loop_end.resolve()
    codegen.pop_data(4)

