import pycro.frontend.python.builtins as builtins
from pycro.frontend.python.opcodes import opcode_with_index
from pycro.frontend.python.compiler import MissingGlobalError

@opcode_with_index('LOAD_GLOBAL')
def load_global(state, codegen, index, code, bcc):
    name = state.get_name(index)
    if name == "False":
        codegen.push_data(0)
    elif name == "True":
        codegen.push_data(1)
    elif name == "AssertionError":
        codegen.push_data(0)
    elif builtins.has_builtin(name):
        # We have a builtin of this name, so add a reference to the builtin version
        builtins.get_builtin(name).add_reference(codegen)
    else:
        state.get_global_variable(name, None).push() 
 
@opcode_with_index('STORE_GLOBAL')
def store_global(state, codegen, index, code, bcc):
    name = state.get_name(index)
    state.get_global_variable(name, None).pop()

@opcode_with_index('STORE_NAME')
def store_name(state, codegen, index, code, bcc):
    """ Used to store a global variable when a module is loading """
    name = state.get_name(index)
    state.get_global_variable(name, None).pop()

@opcode_with_index('LOAD_NAME')
def load_name(state, codegen, index, code, bcc):
    """ Used to load a global variable when a module is loading """
    load_global(state, codegen, index, code, bcc)
