import dis

from pycro.frontend.python.opcodes import opcode_with_index

@opcode_with_index('JUMP_ABSOLUTE')
def absolute_jump(state, codegen, offset, code, bcc):
    state.get_label(offset).jump()

@opcode_with_index('JUMP_IF_TRUE')
def jump_if_true(state, codegen, delta, code, bcc):
    _conditional_jump(state, codegen, delta, code, bcc, True)

@opcode_with_index('JUMP_IF_FALSE')
def jump_if_false(state, codegen, delta, code, bcc):
    _conditional_jump(state, codegen, delta, code, bcc, False)

@opcode_with_index('JUMP_FORWARD')
def jump(state, codegen, delta, code, bcc):
    absolute_jump(state, codegen, bcc + 3 + delta, code, bcc)

def _conditional_jump(state, codegen, delta, code, bcc, jump_if_true):
    # In python, jump 'if statement' don't pop the item
    # from the top of the stack (whereas ours do). That said,
    # it is extremely common to simply pop the result immediately
    # after the jump.
    #
    # Given this, we'll only handle jumps that immediately
    # pop the result. Handling the generic case wouldn't be too
    # difficult (simply push the top of the stack on again).
    assert code.peek(0) == dis.opname.index('POP_TOP')
    assert code.peek(delta) == dis.opname.index('POP_TOP')
    code.remove(delta)
    code.remove(0)
    label = state.get_label(bcc + 3 + delta)

    if jump_if_true:
        label.jump_if_true()
    else:
        label.jump_if_false()

