import dis

from pycro.frontend.python.opcodes import opcode
from pycro.frontend.python.builtins import builder
from pycro.backend.function import Call
import pycro.backend.operations.string as string
from pycro.backend.variable import StackVariable as Variable

@builder(["text"])
def print_item(codegen, function, variables):
    variables[0].push()
    string.print_string(codegen)  

    # All functions need to return a value...
    codegen.push_data(0)
    function.ret()

@opcode('PRINT_ITEM')
def print_opcode(state, codegen, code, bcc):
    # As we need to push the function before the string, we'll push the function,
    # then push a copy of the string pointer.
    text = Variable(codegen, 0)
    print_item.add_reference(codegen)
    text.push()
    Call(codegen).call(1)
    # Pop the result of the print call, and pop the item to be printed (as we
    # had to make a copy of it)
    codegen.pop_data()
    codegen.pop_data()

    # If we aren't printing a new-line next, we should add some whitespace...
    if code.peek(0) != dis.opname.index('PRINT_NEWLINE'):
        print_item.add_reference(codegen)
        string.add_string_reference(codegen, " ")
        Call(codegen).call(1)
        codegen.pop_data()

@opcode ('PRINT_NEWLINE')
def print_newline(state, codegen, code, bcc):
    print_item.add_reference(codegen)
    string.add_string_reference(codegen, "\n")
    Call(codegen).call(1)
    codegen.pop_data()
