#!env python
import logging
import unittest
from pycro.frontend.python.opcodes.constants import BuiltinFunctionError
from pycro.frontend.python.compiler import MissingModuleError
from pycro.frontend.python.test.run import run

class TestConstants(unittest.TestCase):
    def test_string_constant(self):
        """ Test that we can reference a string (even if we don't use it) """
        def bob():
            a = "chicken"
            return 5
        self.assertEqual(5, run(bob))

    def test_sys_exit(self):
        """ Test that we can import sys.exit & call it """
        def bob():
            from sys import exit
            i = 3
            exit(i)
        self.assertEqual(3, run(bob))

    def test_unimplemented_builtin_function(self):
        code = "from sys import unknown_builtin_function"
        logging.disable(logging.CRITICAL)
        self.assertRaises(BuiltinFunctionError, run, compile(code, "<test>", "exec"))
        logging.disable(logging.DEBUG)

    def test_unimplemented_module(self):
        code = "from unknown_module import some_func"
        self.assertRaises(MissingModuleError, run, compile(code, "<test>", "exec"))

if __name__ == "__main__":
    unittest.main()
