#!env python
import unittest
from pycro.frontend.python.test.run import run

class TestFunction(unittest.TestCase):
    def test_embedded_call(self):
        """ Test that we can call a local function """
        def bob():
            def cat():
                return 3
            return cat()

        self.assertEqual(3, run(bob))

    def test_call_logic(self):
        """ Test the a local function's logic is correctly run """
        def bob():
            def cat():
                i = 7
                if i > 2:
                    return 3
                return i
            return cat()
        self.assertEqual(3, run(bob))

    def test_argument(self):
        """ Test passing of simple arguments """
        def bob():
            def cat(i):
                return i + 1
            return cat(1)
        self.assertEqual(2, run(bob))

    def test_multi_args(self):
        """ Test that we can call a function with multiple arguments """
        def bob():
            def cat(i, j):
                return i * j
            return cat(7, 3)
        self.assertEqual(21, run(bob))

    def test_function_as_param(self):
        """ Test passing a function as an argument """
        def bob():
            def cat():
                return 10
            def dog(blah):
                return blah() * blah()
            return dog(cat)
        self.assertEqual(100, run(bob))

    def test_unused_return_value(self):
        """ Test that we successfully ignore return values """
        def bob():
            def cat():
                return 3
            cat()
            return 5
        self.assertEqual(5, run(bob))

if __name__ == "__main__":
    unittest.main()
