#!env python
import logging
import unittest
from pycro.frontend.python.test.run import run
from pycro.frontend.python.compiler import MissingGlobalError

class TestGlobals(unittest.TestCase):
    def test_global_call(self):
        """ Test the calling of a handled builtin function (eg: 'len') """
        def bob():
            return len("chicken")
        self.assertEqual(7, run(bob))

    def test_global_function_copy(self):
        """ Test that we can copy global function objects """
        def bob():
            def cat(func):
                return func("stew")
            return cat(len)
        self.assertEqual(4, run(bob))

    def test_global_variable(self):
        code = """
i = 0;
def bob():
    global i
    i = i + 1
    return i;
def cat():
    return bob();
from sys import exit
exit(bob() + cat())"""
        self.assertEqual(3, run(compile(code, "<test>", "exec")))

    def test_compile_fails_with_missing_global(self):
        """ Test that the compile fails when we reference a missing global """
        code = """
def bob():
    global i
    b = i"""

        logging.disable(logging.CRITICAL)
        self.assertRaises(MissingGlobalError, run, compile(code, "<test>", "exec"))
        logging.disable(logging.DEBUG)

if __name__ == "__main__":
    unittest.main()
