#!env python
import unittest
from pycro.frontend.python.test.run import run, output

class TestList(unittest.TestCase):
    def test_length(self):
        def bob():
            from sys import exit
            if len([]) != 0:
                exit(1)
            if len([1]) != 1:
                exit(2)
            if len([1, 2]) != 2:
                exit(3)
            exit(0)
        self.assertEqual(0, run(bob)) 

    def test_get_item(self):
        def bob():
            from sys import exit
            i = [5, 2, "chicken", 8]
            exit(i[1])
        self.assertEqual(2, run(bob))

    def test_string_in_list(self):
        def bob():
            from sys import exit
            i = [5, 2, "chicken", 8]
            print i[2]
        self.assertEqual("chicken\n", output(bob))

    def test_set_item(self):
        def bob():
            from sys import exit
            i = [5, 7]
            i[1] = 1
            exit(i[0] - i[1])
        self.assertEqual(4, run(bob))

    def test_list_in_list(self):
        def bob():
            from sys import exit
            i = [[5, 2], [7, 1]]
            exit(i[0][0] - i[1][1])
        self.assertEqual(4, run(bob))

    def test_append_length(self):
        def bob():
            i = [4]
            i.append(5)
            return len(i)
        self.assertEqual(2, run(bob))

    def test_append_value(self):
        def bob():
            i = [4]
            i.append(5)
            return i[1]
        self.assertEqual(5, run(bob))

    def test_append_multiple_values(self):
        def bob():
            x = []
            i = 0
            while i < 300:
                x.append(i)
                i = i + 1
            return x[299] - x[1]
        self.assertEqual(298 & 0xff, run(bob))

    def test_empty_reverse(self):
        def bob():
            x = []
            x.reverse()
            return len(x) + 1
        self.assertEqual(1, run(bob))

    def test_odd_reverse(self):
        def bob():
            x = [1,2,3]
            x.reverse()
            if x[0] != 3:
                return 5
            if x[1] != 2:
                return 5
            if x[2] != 1:
                return 5
            return 7
        self.assertEqual(7, run(bob))

    def test_insert(self):
        def bob():
            x = [3,8]
            x.insert(1, 3)
            return x[2] - x[1]
        self.assertEqual(5, run(bob))
            
    def test_length_after_insert(self):
        def bob():
            x = [3,8]
            x.insert(1, 3)
            return len(x)
        self.assertEqual(3, run(bob))

if __name__ == "__main__":
    unittest.main()

