import os
from pycro.frontend.python import default_config
import pycro.frontend.python.compiler as compiler
import pycro.backend.test.comfile as comfile
import pycro.backend.codegen as cgen
import pycro.backend.function as function
from pycro.backend.lib.sys import exit as sys_exit

def execute(func, include_path=[]):
    """ Convert a function to a program, returning tuple containing 
        the program's exit code & standard output.  """
    
    codegen = cgen.CodeGen()
    config = default_config()
    config.python.include_path.extend(include_path)

    state = compiler.Compiler(codegen, config)

    if hasattr(func, "func_code"):
        # We are decoding a function (as opposed to a code block).
        assert func.func_code.co_argcount == 0
        func = func.func_code

    state.decode_module(func)
    return comfile.run(codegen.create_output())

def output(func, include_path=[]):
    """ Convert the function to a program, returning the standard
        output. """
    return execute(func, include_path)[1]

def run(func, include_path=[]):
    """ Convert the function to a program, returing the exit code. """
    return execute(func, include_path)[0]
