#!env python

import unittest

from pycro.backend.codegen import CodeGen
from pycro.backend.test.comfile import run
from pycro.frontend.python.compiler import Compiler
import pycro.frontend.python.test.run

def _run(text):
    
    codegen = CodeGen()
    compiler = Compiler(codegen)

    code = compile(text, "<string>", "exec")
    compiler.decode_module(code)
    return run(codegen.create_output())

class TestCompiler(unittest.TestCase):
    def test_module(self):
        code = """i = 0
while i < 5:
    i = i + 1"""
        self.assertEqual((0, ""), _run(code))

    def test_simple_print(self):
        code = """i = 0
while i < 5:
    print "hello"
    i = i + 1"""
        self.assertEqual((0, "hello\n" * 5), _run(code))

    def test_empty_module(self):
        # Modules in python have a default 'return 0' when they are 
        # implemented. Test that this is turned into an exit code.
        self.assertEqual((0, ""), _run(""))

if __name__ == "__main__":
    unittest.main()
