#!env python
import getopt
import os.path
import sys
import unittest

_ignore_dirs = ["samples"]

def _load_all_tests(root, filter):
    # 'root' may not be in the current working directory...
    sys.path.append(root)

    root = os.path.normpath(root)
    suite = unittest.TestSuite()
    for (dirpath, dirnames, filenames) in os.walk(root):
        for sub_dir in dirnames[:]:
            if sub_dir in _ignore_dirs:
                dirnames.remove(sub_dir)

        dirpath = os.path.normpath(dirpath)[len(root) + 1:]
        modules = os.path.normpath(dirpath).split(os.sep)
        if modules[-1] == "test":
            for filename in filenames:
                (path_root, ext) = os.path.splitext(filename)
                if ext == ".py" and path_root.startswith('test'):
                    component = modules + [path_root]
                    module_name = '.'.join(component)
                    if module_name.startswith(filter):
                        module = __import__(module_name)
                        for comp in component[1:]:
                            module = getattr(module, comp)
                        suite.addTest(unittest.defaultTestLoader.loadTestsFromModule(module))
    return suite

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "v", ["verbose"])
    except getopt.GetoptError, ex:
        sys.stderr.write(str(ex) + "\n")
        sys.exit(2)

    verbose = 1
    for o, a in opts:
        if o in ["-v", "--verbose"]:
            verbose = 2

    filter = ""
    if len(args) == 1:
        filter = args[0]
    elif len(args) > 1:
        sys.stderr.write("Only a single filter can be used! (Filters = %s)" % ", ".join(args))
        sys.exit(2)

    suite = _load_all_tests(os.getcwd(), filter)
    unittest.TextTestRunner(verbosity=verbose).run(suite)

if __name__ == '__main__':
    main()
