#!env python

import unittest

import common.graphics as graph
import common.graphics.triangle as tri

class TestTriangle(unittest.TestCase):
    def setUp(self):
        graph._reset_scanline()

    def check_line_bounds(self, y, start, end):
        self.assertEqual(start, graph._scanlines[y][graph.NEXT_SECTION][graph.X])
        self.assertEqual(end, graph._scanlines[y][graph.NEXT_SECTION][graph.NEXT_SECTION][graph.X])

    def is_empty(self, y):
        return (0 == graph._scanlines[y][graph.X]) and \
            (320 == graph._scanlines[y][graph.NEXT_SECTION][graph.X])

    def check_bounds(self, minx, miny, maxx, maxy):
        """ Check that all drawn lines are between a given section """
        for y in range(0, miny):
            self.assertTrue(self.is_empty(y))
        for y in range(miny, maxy):
            self.assertFalse(self.is_empty(y))
            self.assertTrue(minx <= graph._scanlines[y][graph.NEXT_SECTION][graph.X])
            self.assertTrue(maxx >= graph._scanlines[y][graph.NEXT_SECTION][graph.NEXT_SECTION][graph.X])
        for y in range(maxy, graph._res_y):
            self.assertTrue(self.is_empty(y))

    def test_max_y_equals(self):
        tri.draw_triangle(10, 5, 0, 20, 15, 0, 8, 15, 0, 3)
        # Make sure the final line isn't included
        self.assertTrue(self.is_empty(15))
        self.check_line_bounds(14, 8, 19)
        self.check_bounds(8, 6, 20, 15)

    def test_blah(self):
        tri.draw_triangle(140, 30, 0, 179, 30, 0, 179, 69, 0, 3)
        self.check_bounds(140, 30, 179, 69)

if __name__ == "__main__":
    unittest.main()
