from common.graphics import add_span
#def add_span(y, x1, x2, colour):
#    print "y:%i x1:%i x2:%i" % (y, x1, x2)

"""
Triangle renderer
=================

This is the scanline triangle renderer. The renderer is 'left inclusive'; eg:
a triangle includes the left and top pixels, but not the right or bottom edge
pixels.
"""

# The number of bits we'll use to keep track of sub-integer precision
_precision = 6
_prec_factor = 64

def _add_section(x, y, z, x2, y2, z2, x3, y3, z3, colour):
    # If the two 'endpoints' are in different directions, then
    # there are no start and end points to be drawn.
    is_out_of_bounds = 0
    if y > y2:
        is_out_of_bounds = y < y3
    if y < y2:
        is_out_of_bounds = y > y3
    if is_out_of_bounds:
        return

    if y == y2:
        return
    if y == y3:
        return

    s1 = ((x2 - x) * _prec_factor) / (y2 - y)
    s2 = ((x3 - x) * _prec_factor) / (y3 - y)

    if z == z2:
        z1 = 0
    else:
        z1 = ((z2 - z) * _prec_factor) / (z2 - z)

    if z == z3:
        z2 = 0
    else:
        z2 = ((z3 - z) * _prec_factor) / (z3 - z)

    if y > y2:
        increment = -1
        end = max(y2, y3) - 1

        temp = s1
        s1 = 0 - s2
        s2 = 0 - temp

        temp = z1
        z1 = 0 - z2
        z2 = 0 - temp
    if y < y2:
        increment = 1
        end = min(y2, y3)

    left = x * _prec_factor
    right = left

    zleft = z * _prec_factor
    zright = zleft
    while y != end:
        add_span(y, left  / _prec_factor, zleft / _prec_factor, right / _prec_factor, zright / _prec_factor, colour)
        left = left + s2
        right = right + s1
        zleft = zleft + z2
        zright = zright + z1
        y = y + increment

def draw_triangle(x1, y1, z1, x2, y2, z2, x3, y3, z3, colour):
    _add_section(x1, y1, z1, x2, y2, z2, x3, y3, z3, colour)
    _add_section(x2, y2, z2, x3, y3, z3, x1, y1, z1, colour)
    _add_section(x3, y3, z3, x1, y1, z1, x2, y2, z2, colour)

