#!env python
from system.input import peek, readkey
from system.drawing.window import init, close

from common.graphics import swap
from common.graphics.renderer import render, set_world_transform
from common.matrix import create_matrix, multiply, make_identity
from common.math import sin, cos

_vertices = None
_triangles = None
def _make_cube():
    global _vertices, _triangles
    _vertices = [[-10, 10,-10], [10, 10,-10], [10,-10,-10], [-10,-10,-10],
                 [-10, 10, 10], [10, 10, 10], [10,-10, 10], [-10,-10, 10]]
    _triangles = [[0, 1, 2], [0, 2, 3], [0, 4, 5], [0, 5, 1],
                  [1, 5, 6], [1, 6, 2], [2, 6, 7], [2, 7, 3],
                  [3, 7, 4], [3, 4, 0], [4, 7, 6], [4, 6, 5]]
_make_cube()

def _get_cube_positions():
    result = []
    colour = 1
    i = 4
    while i >= -4:
        j = -4
        while j <= 4:
            k = -4
            while k <= 4:
                cube = [j * 10, k * 10, i * 10, colour]
                result.append(cube)
                k = k + 4
            j = j + 4
        i = i - 4
    return result

_view_rotate = create_matrix()
_view_offset = create_matrix()
def _update_view_transform(i, eye_z, matrix):
    # Rotate the position around the y-axis
    _view_offset[0][3] = (100 + eye_z) * sin(i) / 100
    _view_offset[2][3] = (100 + eye_z) * cos(i) / 100

    # Look back towards the origin
    _view_rotate[0][0] = cos(i)
    _view_rotate[0][2] = 0-sin(i)
    _view_rotate[1][1] = 100
    _view_rotate[2][0] = sin(i)
    _view_rotate[2][2] = cos(i)
    _view_rotate[3][3] = 100

    multiply(_view_rotate, _view_offset, matrix)

def main():
    init()

    world = create_matrix()
    view = create_matrix()

    cubes = _get_cube_positions()
    final_transform = create_matrix()

    should_run = 1
    i = 0
    eye_z = 0
    while should_run:
        _update_view_transform(i, eye_z, view)
        i = (i + 1) % 360

        for cube in cubes:
            world[0][3] = cube[0]
            world[1][3] = cube[1]
            world[2][3] = cube[2]
            colour = cube[3]

            multiply(view, world, final_transform)
            set_world_transform(final_transform)
            render(_vertices, _triangles, colour)

        if peek():
            key = readkey()
            ESC = 27
            if key == ESC:
                should_run = 0

            if key == 0:
                # Special key...
                UP_ARROW = 72
                DOWN_ARROW = 80
                key = readkey()
                if key == UP_ARROW:
                    eye_z = eye_z + 1
                elif key == DOWN_ARROW:
                    eye_z = eye_z - 1

        swap()
    close()
main()
